"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.endGroup = exports.startGroup = exports.entitle = exports.areGroupsEnabled = exports.grouping = exports.disableConsoleGroups = exports.useConsoleGroups = void 0;
const ansi_colors_1 = require("ansi-colors");
const util = require("util");
const get_current_line_1 = require("get-current-line");
// eslint-disable-next-line @typescript-eslint/no-unused-vars
console.group = (..._) => undefined;
console.groupEnd = () => undefined;
const originalLog = console.log;
const originalInfo = console.info;
const originalWarn = console.warn;
const originalError = console.error;
const originalGroup = console.group;
const originalGroupEnd = console.groupEnd;
let groupsEnabled = false;
function useConsoleGroups() {
    groupsEnabled = true;
    console.log = consoleLog;
    console.info = consoleInfo;
    console.warn = consoleWarn;
    console.error = consoleError;
    console.group = consoleGroup;
    console.groupEnd = consoleGroupEnd;
}
exports.useConsoleGroups = useConsoleGroups;
function disableConsoleGroups() {
    groupsEnabled = false;
    console.log = originalLog;
    console.warn = originalWarn;
    console.error = originalError;
    console.group = originalGroup;
    console.groupEnd = console.groupEnd = originalGroupEnd;
    console.info = originalInfo;
}
exports.disableConsoleGroups = disableConsoleGroups;
const themes = {
    asciiTheme: { prefix: '│', titlePrefix: '├ ', spacing: 1 },
    cleanTheme: { prefix: ' ', titlePrefix: '', spacing: 1 },
};
const theme = themes.cleanTheme;
const cutLength = () => {
    const { prefix, spacing } = theme;
    return prefix.length + spacing;
};
let prefix = '';
function format(...args) {
    const asString = util.format(...args);
    return asString.replace(/^/gm, prefix);
}
function formatColor(color, prepad, ...args) {
    const asString = util.format(...args);
    const prepadding = ' '.repeat(prepad);
    return color(asString).replace(/^/gm, prefix + prepadding);
}
function writeToStream(name, color, ...args) {
    const { framesToSkip } = getFramesToSkip();
    const { line, char, file } = (0, get_current_line_1.default)(framesToSkip);
    const callerString = (0, ansi_colors_1.bold)(`${file}:${line}:${char}`);
    const newLine = (arg) => `${arg}\n`;
    process.stdout.write(newLine(formatColor(color, 1, (0, ansi_colors_1.bold)(`[${name}]`))));
    process.stdout.write(newLine(formatColor(color, 1, ...args)));
    process.stdout.write(newLine(formatColor(color, 1, callerString)));
    process.stdout.write(newLine(format('')));
}
function writeToErr(name, color, ...args) {
    const { framesToSkip } = getFramesToSkip();
    const { line, char, file } = (0, get_current_line_1.default)(framesToSkip);
    const callerString = (0, ansi_colors_1.bold)(`${file}:${line}:${char}`);
    const newLine = (arg) => `${arg}\n`;
    process.stderr.write(newLine(formatColor(color, 1, (0, ansi_colors_1.bold)(`[${name}]`))));
    process.stderr.write(newLine(formatColor(color, 1, ...args)));
    process.stderr.write(newLine(formatColor(color, 1, callerString)));
    process.stderr.write(newLine(format('')));
}
function getFramesToSkip() {
    const framesToSkip = 2;
    return {
        framesToSkip: {
            method: 'getCurrentLine',
            frames: framesToSkip,
            immediate: false,
        },
    };
}
function consoleLog(...args) {
    writeToStream('Log', ansi_colors_1.white, ...args);
}
function consoleError(...args) {
    writeToErr('Error', ansi_colors_1.red, ...args);
}
function consoleWarn(...args) {
    writeToErr('Warn', ansi_colors_1.yellow, ...args);
}
function consoleInfo(...args) {
    writeToStream('Info', ansi_colors_1.blackBright, ...args);
}
function consoleGroup(...args) {
    const { prefix: themePrefix, titlePrefix, spacing } = theme;
    process.stdout.write('\u001b[1m' + format(titlePrefix + args.join(' ')) + '\u001b[22m\n');
    prefix += themePrefix + ' '.repeat(spacing);
}
function consoleGroupEnd() {
    prefix = prefix.slice(0, -cutLength());
}
const groupStack = [];
function grouping(title, action) {
    console.group(title);
    try {
        return action();
    }
    catch (e) {
        throw new Error('grouping failed due to action throwing an error ' + e);
    }
    finally {
        console.groupEnd();
    }
}
exports.grouping = grouping;
function areGroupsEnabled() {
    return groupsEnabled;
}
exports.areGroupsEnabled = areGroupsEnabled;
function entitle(title) {
    if (groupsEnabled) {
        process.stdout.write('\u001b[1m' + format(title) + '\u001b[22m\n');
    }
}
exports.entitle = entitle;
function startGroup(type, ...tags) {
    if (groupsEnabled) {
        groupStack.push(type);
        console.group([type, ...tags].join(' ').trim());
    }
}
exports.startGroup = startGroup;
function endGroup(type) {
    if (groupsEnabled) {
        const peek = groupStack.at(-1);
        if (peek !== type) {
            console.warn(`Attempting to end console group '${type}', however currently active group is '${peek}'. Make sure you end any open inner groups, and beware asynchronous grouping.`);
        }
        console.groupEnd();
        groupStack.pop();
    }
}
exports.endGroup = endGroup;
//# sourceMappingURL=console.js.map